%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SCRIPT ZamudioA9
% Prepares data and calls routines for Truncated regression.
% 
%                                         by Csar Zamudio, 11/16/2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;clc;

%Set format
format short;

%Parameters for analysis
trueb=1;

%Number of simulations
nsim=200;

%Declare space to gather degree of truncation.
sizes=zeros(nsim,2);
betas200=zeros(nsim,3);
betas500=zeros(nsim,3);
vbetas200=zeros(nsim,3);
vbetas500=zeros(nsim,3);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Q1. Monte Carlo study
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%We want to show the mean of beta mle using the regression model
%yi=a+xi+ei using truncated data. We will use N=200 and N=500 in the DGP.
%Since y* may <=0, and those observations should be unobserved, then only
%observations for which yi*>0 will be retained. This is the truncation.
disp('====ANSWER TO QUESTION 1: Monte Carlo study=====');
fprintf('\n')
%Monte Carlo simulation starts
%Loop for N=200, 500
for i=1:1:2
    %Choose N
    if i==1
        N=200;
    else
        N=500;
    end
    
    for S=1:1:nsim
        disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
        disp('Q1. Monte Carlo study: Truncated data')
        disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
        fprintf('\n');
        disp('Now calculating iteration...');
        [N S]
        %Generate simulated data many times (Monte Carlo)
        %Recognize that data is ALREADY truncated.
        data=DGP(N,trueb);
        Y=data(:,1); X=data(:,2);
        N2=size(X,1);
        X=[ones(N2,1) X];
        %Gather degree of truncation of each Monte Carlo sample.
        sizes(S,i)=N-size(Y,1);
        %Perform OLS estimation
        bOLS=inv(X'*X)*X'*Y;
        %Store beta
        if i==1
            betas200(S,1)=bOLS(2,1);        
        else
            betas500(S,1)=bOLS(2,1);        
        end
        res=Y-X*bOLS;
        v=(sum(res.^2)/(N2-2));
        vOLS=(sum(res.^2)/(N2-2))*inv(X'*X);
        vOLS=diag(vOLS);
        %Store variance
        if i==1
            vbetas200(S,1)=vOLS(2,1);
        else
            vbetas500(S,1)=vOLS(2,1);
        end
                
        %Perfom MLE estimation                
        betainit=[bOLS;v]; %a,b,sigma
        options= optimset('Display', 'off','LargeScale','off', 'HessUpdate', 'dfp');
        [beta,fval,exitflag,output,grad,hessian2]=fminunc(@(x) OLSMLE(x,Y,X,N2),betainit,options);
        if i==1
            betas200(S,2)=beta(2,1);
            vv=(diag(inv(hessian2)));
            vbetas200(S,2)=vv(2,1);
        else
            betas500(S,2)=beta(2,1);
            vv=(diag(inv(hessian2)));
            vbetas500(S,2)=vv(2,1);
        end
          
        
        %Perfom MLE estimation WITH TRUNCATION CORRECTION!
        betainit=[bOLS;v]; %a,b,sigma        
        options= optimset('Display', 'off','LargeScale','off', 'HessUpdate', 'dfp');
        [beta,fval,exitflag,output,grad,hessian2]=fminunc(@(x) OLSMLE_Trunc(x,Y,X,N2),betainit,options);                
        if i==1
            betas200(S,3)=beta(2,1);
            vv=(diag(inv(hessian2)));
            vbetas200(S,3)=vv(2,1);
        else
            betas500(S,3)=beta(2,1);
            vv=(diag(inv(hessian2)));
            vbetas500(S,3)=vv(2,1);
        end
        pause(.001)
        clc
    end
end

disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('Q1. Monte Carlo study: Truncated data')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
fprintf('\n');
disp('All simulations done!')
fprintf('\n')
resultsB=[sum(betas200(:,1))/nsim sum(betas200(:,2))/nsim sum(betas200(:,3))/nsim; ...
          sum(betas500(:,1))/nsim sum(betas500(:,2))/nsim sum(betas500(:,3))/nsim];
resultsV=[sum(vbetas200(:,1))/nsim sum(vbetas200(:,2))/nsim sum(vbetas200(:,3))/nsim; ...
          sum(vbetas500(:,1))/nsim sum(vbetas500(:,2))/nsim sum(vbetas500(:,3))/nsim];
resultsV=real(resultsV);
%Display summary of results using "displaytable" function from Mathworks.
displaytable(resultsB,{'',''},{{'N=200','N=500'},{'OLS' 'OLS_MLE' 'OLS_MLE, TRUNCATED'}},25,4)
fprintf('\n')
displaytable(resultsV,{'',''},{{'N=200','N=500'},{'V(OLS)' 'V(OLS_MLE)' 'V(OLS_MLE, TRUNCATED)'}},25,4)
fprintf('\n')
disp('As can be seen, as N grows (200->500) there is no effect on the mean  estimated.');
disp('However, using the TRUNCATED MLE instead of ignoring truncation makes the estimated  ');
disp('be VERY close to the true value, which is =1.');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Q2. Powell's trimmed estimator
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
betas200=zeros(nsim,1);
betas500=zeros(nsim,1);
vbetas200=zeros(nsim,1);
vbetas500=zeros(nsim,1);

fprintf('\n')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('Q2. Monte Carlo study: POWELL trimmed estimator')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
fprintf('\n');
disp('Now calculating iteration...');
fprintf('\n');
disp('All simulations done!')
fprintf('\n')
%We do exactly the same as above but using Powell's trimmed estimator.

%Monte Carlo simulation starts
%Loop for N=200, 500
for i=1:1:2
    %Choose N
    if i==1
        N=200;
    else
        N=500;
    end
    
    for S=1:1:nsim           
        %Generate simulated data many times (Monte Carlo)
        %Recognize that data is ALREADY truncated.
        data=DGP(N,trueb);
        Y=data(:,1); X=data(:,2);
        N2=size(X,1);
        X=[ones(N2,1) X];
        %Gather degree of truncation of each Monte Carlo sample.
        sizes(S,i)=N-size(Y,1);
        %Perform OLS estimation for starting values
        bOLS=inv(X'*X)*X'*Y;        
                
        %Perfom MLE estimation                
        betainit=[bOLS]; %a,b,sigma
        options= optimset('Display', 'off','LargeScale','off', 'HessUpdate', 'dfp');
        [beta,fval,exitflag,output,grad,hessian2]=fminunc(@(x) POWELL(x,Y,X,N2),betainit,options);
        if i==1
            betas200(S,1)=beta(2,1);
            vv=(diag(inv(hessian2)));
            vbetas200(S,1)=vv(2,1);
        else
            betas500(S,1)=beta(2,1);
            vv=(diag(inv(hessian2)));
            vbetas500(S,1)=vv(2,1);
        end                          
    end
    
end

resultsBPOW=[sum(betas200)/nsim; sum(betas500)/nsim];
resultsVPOW=[sum(vbetas200)/nsim; sum(vbetas500)/nsim];
fullResB=[resultsB(1,3) resultsBPOW(1,1); resultsB(2,3) resultsBPOW(2,1)];
fullResV=[resultsV(1,3) resultsVPOW(1,1); resultsV(2,3) resultsVPOW(2,1)];
fullResV=real(fullResV);
%Display summary of results using "displaytable" function from Mathworks.
displaytable(fullResB,{'',''},{{'N=200','N=500'},{'OLS_MLE, TRUNCATED' 'POWELL'}},25,4)
fprintf('\n')
displaytable(fullResV,{'',''},{{'N=200','N=500'},{'V(OLS_MLE, TRUNCATED)' 'POWELL'}},25,4)
fprintf('\n')
disp('As shown, the POWELL estimator performs better than a simple MLE truncation.');
